Mac OS X Build Instructions and Notes
=====================================

IMPORTANT: Compilers and libraries have subtle differences among platforms.
Please note that builds created on Mac OS are currently not extensively
tested by QA team and are therefore not in scope for bug bounty programs.
Provided instructions are intended for developers that would like to use
Mac OS development tools.

### Prerequirements

1.  Install Xcode from the app store

2.  Install the OS X command line tools:

`xcode-select --install`

When the popup appears, click `Install`.

3.  Install [Homebrew](http://brew.sh).

### Installing dependencies

    brew install automake berkeley-db libtool boost openssl pkg-config libevent

### Cloning Bitcoin SV

        git clone https://github.com/bitcoin-sv/bitcoin-sv

### Building Bitcoin SV - Autotools

    cd bitcoin-sv
    ./autogen.sh
    ./configure
    make

### Building Bitcoin SV - CMake

    mkdir build_bitcoin_sv
    cd build_bitcoin_sv
    cmake -D CMAKE_BUILD_TYPE=RelWithDebInfo ../bitcoin-sv
    make -j4

    # building unit tests
    make test_bitcoin -j4

    # Xcode project can be generated by providing an additional flag to cmake
    # command above:
    # cmake -G Xcode -D CMAKE_BUILD_TYPE=RelWithDebInfo ../bitcoin-sv
    #
    # Alternatively -G "Sublime Text 2 - Unix Makefiles" generates a
    # Sublime text 3 project

### Potential python core dumps while running functional tests

There is a chance on MacOS that functional tests start producing python
core dumps due to test/functional/test_framework/key.py using invalid OpenSSL
library.
This can happen due to conflicting loads of system and Homebrew/MacPorts OpenSSL
libraries caused by MacOS System Integrity Protection not allowing to override
library loading paths from outside world.

In such case as a workaround test/functional/test_framework/key.py should be
modified by changing the library search path and modifying the value of ssl
variable:

    import os
    import platform
    import subprocess

    search_path = ['/opt/local/lib']
    try:
        homebrew_libdir = subprocess.check_output(['brew', '--prefix', 'openssl'])
        homebrew_libdir = homebrew_libdir.decode('UTF-8').rstrip('\n')
        search_path.insert(0, os.path.join(homebrew_libdir, 'lib'))
    except subprocess.SubprocessError:
        pass
    dyld_library_path = os.environ.get('DYLD_LIBRARY_PATH', None)
    if dyld_library_path:
        search_path.append(dyld_library_path.lstrip(os.path.pathsep))
    os.environ['DYLD_LIBRARY_PATH'] = os.path.pathsep.join(search_path)

    ssl = ctypes.cdll.LoadLibrary(   ctypes.util.find_library('ssl')
                                  or ctypes.util.find_library('libcrypto-1_1-x64')
                                  or ctypes.util.find_library('libeay32'))